#include "StdAfx.h"
#include "AutoTextListCtrl.h"

CAutoTextListCtrl::CAutoTextListCtrl(void)
{
	m_crNormalText		= COLOR_LC_NORMAL_TEXT;
	m_crNormalBk		= COLOR_LC_NORMAL_BK;
	m_crSelectedText	= COLOR_LC_SELECTED_TEXT;
	m_crSelectedBk		= COLOR_LC_SELECTED_BK;
	m_crLinkNormal		= COLOR_LC_LINK_NORMAL_TEXT;
	m_crLinkHover		= COLOR_LC_LINK_HOVER_TEXT;

	m_nCursor			= CURSOR_DEFAULT;

	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));       
	g_ftDefault8PT.GetLogFont(&lf);
	m_fontText.CreateFontIndirect(&lf);

}

CAutoTextListCtrl::~CAutoTextListCtrl(void)
{
	m_fontText.DeleteObject();
}

BEGIN_MESSAGE_MAP(CAutoTextListCtrl, CListCtrl)
	ON_WM_MEASUREITEM_REFLECT()
	ON_WM_SETCURSOR()
	ON_NOTIFY_REFLECT(NM_CUSTOMDRAW, &CAutoTextListCtrl::OnNMCustomdraw)
	ON_MESSAGE(LVM_SETCOLUMNWIDTH, &CAutoTextListCtrl::OnSetColumnWidth)
END_MESSAGE_MAP()

int CAutoTextListCtrl::GetColumnCount()
{
	return GetHeaderCtrl()->GetItemCount();
}

void CAutoTextListCtrl::MeasureItem(LPMEASUREITEMSTRUCT lpMeasureItemStruct)
{
	// Listctrl  ̸ ϱ  ߰Ͽ,  ʾ Font ̿  Ͽ
	LOGFONT lf;
	GetFont()->GetLogFont( &lf );
 
	if (lf.lfHeight < 0)
		lpMeasureItemStruct->itemHeight = -lf.lfHeight + 10; 
	else
		lpMeasureItemStruct->itemHeight = lf.lfHeight + 10; 
}

void CAutoTextListCtrl::OnNMCustomdraw(NMHDR* pNMHDR, LRESULT* pResult)
{
	NMLVCUSTOMDRAW* pLVCD = reinterpret_cast<NMLVCUSTOMDRAW*>(pNMHDR);

	int nItem = static_cast<int> (pLVCD->nmcd.dwItemSpec);
	int nSubItem = pLVCD->iSubItem;
	//Trace(_T("(%d,%d) %d,%d,%d,%d\n"), nItem, nSubItem, pLVCD->rcText.left, pLVCD->rcText.right, pLVCD->rcText.top, pLVCD->rcText.bottom);

	*pResult = CDRF_DODEFAULT;

	if (pLVCD->nmcd.dwDrawStage == CDDS_PREPAINT)
	{
		*pResult = CDRF_NOTIFYITEMDRAW;
	}
	else if (pLVCD->nmcd.dwDrawStage == CDDS_ITEMPREPAINT)
	{
		*pResult = CDRF_NOTIFYSUBITEMDRAW;
	}
	else if (pLVCD->nmcd.dwDrawStage == (CDDS_ITEMPREPAINT | CDDS_SUBITEM))
	{
		*pResult = CDRF_SKIPDEFAULT;	// Default Լ  ʰ  ׸

		CDC* pDC = CDC::FromHandle(pLVCD->nmcd.hdc);
		DrawSubItem(nItem, nSubItem, pDC);
	}
}

void CAutoTextListCtrl::DrawSubItem(int nItem, int nSubItem, CDC *pDC)
{
	CRect rect(0, 0, 0, 0);
	//GetSubItemRect(nItem, nSubItem, LVIR_BOUNDS, rect);
	GetSubItemRect(nItem, nSubItem, LVIR_LABEL, rect);

	/* LVIR_BOUNDS ϸ Ư 쿡 Ʈ  ̰  ׷ ʴ´.
	 * Ƹ SubItem 0  ش Row ü  ͼ  ĥֱ  ׷  .
	 *   ذϱ  LVIR_LABEL  ϸ ̴  , SubItem 0  left  right  ũ ´.
	 * ׷ SubItem 0ΰ left   ִ  óϿ.
	 */
	if (nSubItem == 0)
		rect.left = 0;

	COLORREF crText;
	COLORREF crBkgnd;

	GetDrawColors(nItem, nSubItem, crText, crBkgnd);
	pDC->FillSolidRect(&rect, crBkgnd);

	CString strText = GetItemText(nItem, nSubItem);

	if (!strText.IsEmpty())
	{
		CFont *pOldFont = pDC->SelectObject(&m_fontText);

		// get text justification
		HDITEM hditem;
		hditem.mask = HDI_FORMAT;
		GetHeaderCtrl()->GetItem(nSubItem, &hditem);

		int nFmt = hditem.fmt & HDF_JUSTIFYMASK;
		UINT nFormat = DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS | DT_NOPREFIX;

		if (nFmt == HDF_CENTER)
		{
			nFormat |= DT_CENTER;
		}
		else if (nFmt == HDF_LEFT)
		{
			nFormat |= DT_LEFT;
			rect.left += 10;
		}
		else
		{
			nFormat |= DT_RIGHT;
		}

		pDC->SetBkMode(TRANSPARENT);
		pDC->SetTextColor(crText);			
		pDC->SetBkColor(crBkgnd);
		pDC->DrawText(strText, &rect, nFormat);
		pDC->SelectObject(pOldFont);
	}
}

void CAutoTextListCtrl::GetDrawColors(int nItem, int nSubItem, COLORREF& colorText, COLORREF& colorBkgnd)
{
	DWORD dwStyle    = GetStyle();
	DWORD dwExStyle  = GetExtendedStyle();

	COLORREF crText  = m_crNormalText;
	COLORREF crBkgnd = m_crNormalBk;

	if (GetItemState(nItem, LVIS_SELECTED))
	{
		crText  = m_crSelectedText;
		crBkgnd = m_crSelectedBk;

		if (m_hWnd != ::GetFocus())	// Ŀ   õ  
		{
			if (dwStyle & LVS_SHOWSELALWAYS)
			{
				crText  = m_crSelectedText;
				crBkgnd = m_crSelectedBk;
			}
			else
			{
				crText  = m_crSelectedText;
				crBkgnd = m_crSelectedBk;
			}
		}
	}

	colorText = crText;
	colorBkgnd = crBkgnd;
}

void CAutoTextListCtrl::SetRowHeight(int nRowHeight)
{
	CFont font;
	LOGFONT logfont;

	CDC *pDC = GetDC();
	if( !pDC ) return;

	int nRevision=7;

	logfont.lfHeight = -MulDiv(nRowHeight-nRevision, GetDeviceCaps(pDC->m_hDC, LOGPIXELSY), 72);
	ReleaseDC( pDC );

	logfont.lfEscapement	= 0;
	logfont.lfOrientation	= 0;
	logfont.lfWeight		= FW_NORMAL;
	logfont.lfItalic		= FALSE;
	logfont.lfUnderline		= FALSE;
	logfont.lfStrikeOut		= FALSE;
	logfont.lfCharSet		= ANSI_CHARSET;
	logfont.lfOutPrecision	= OUT_DEFAULT_PRECIS;
	logfont.lfClipPrecision	= CLIP_DEFAULT_PRECIS;
	logfont.lfQuality		= PROOF_QUALITY;
	logfont.lfPitchAndFamily= DEFAULT_PITCH;
	_tcscpy_s(logfont.lfFaceName, _T("System"));
	font.CreateFontIndirect(&logfont);

	SetFont(&font);

	font.DeleteObject();
}

void CAutoTextListCtrl::SetTextFont(CFont* pFont, BOOL bRedraw)
{
	if (pFont == NULL)
		return;

	m_fontText.DeleteObject();

	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));       
	pFont->GetLogFont(&lf);
	m_fontText.CreateFontIndirect(&lf);

	if (bRedraw)
		RedrawWindow();
}

void CAutoTextListCtrl::SetNormalTextColor(COLORREF crNormalText, COLORREF crNormalBk)
{
	m_crNormalText = crNormalText;
	m_crNormalBk = crNormalBk;
}

void CAutoTextListCtrl::SetSelectedTextColor(COLORREF crSelectedText, COLORREF crSelectedBk)
{
	m_crSelectedText = crSelectedText;
	m_crSelectedBk = crSelectedBk;
}

BOOL CAutoTextListCtrl::SelectAll()
{
	return SetItemState(-1, LVIS_SELECTED, LVIS_SELECTED);
}

BOOL CAutoTextListCtrl::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message)
{
	switch (m_nCursor)
	{
	case CURSOR_HAND:
		SetCursor(AfxGetApp()->LoadStandardCursor(IDC_HAND));
		return TRUE;
	case CURSOR_DEFAULT:
		SetCursor(AfxGetApp()->LoadStandardCursor(IDC_ARROW));
		return TRUE;
	}

	return CListCtrl::OnSetCursor(pWnd, nHitTest, message);
}

LRESULT CAutoTextListCtrl::OnSetColumnWidth(WPARAM wParam, LPARAM lParam)
{
	int nCol = (int)wParam;

	return DefWindowProc(LVM_SETCOLUMNWIDTH, wParam, lParam);
}

void CAutoTextListCtrl::ResizeColumn()
{
	CRect rtClient(0, 0, 0, 0);
	GetClientRect(rtClient);

	int nColWidth = rtClient.Width() - GetColumnWidth(0);

	// 0 ÷ ϰ, 1 ÷  ä.
	SetColumnWidth(1, nColWidth);
}

